@@ -1,8 +1,18 @@
-import os
 import json
-from fortls.objects import fortran_ast, fortran_module, fortran_subroutine, \
-    fortran_function, fortran_type, fortran_var, fortran_obj, map_keywords
-none_ast = fortran_ast()
+import os
+
+from fortls.helper_functions import map_keywords
+from fortls.objects import (
+    FortranAST,
+    FortranObj,
+    Function,
+    Module,
+    Subroutine,
+    Type,
+    Variable,
+)
+
+none_ast = FortranAST()
 lowercase_intrinsics = False
 
 
@@ -11,12 +21,12 @@ def set_lowercase_intrinsics():
     lowercase_intrinsics = True
 
 
-class fortran_intrinsic_obj(fortran_obj):
+class Intrinsic(FortranObj):
     def __init__(self, name, type, doc_str=None, args="", parent=None):
         self.name = name
         self.type = type
         self.doc_str = doc_str
-        self.args = args.replace(' ', '')
+        self.args = args.replace(" ", "")
         self.parent = parent
         self.file_ast = none_ast
         if lowercase_intrinsics:
@@ -28,13 +38,13 @@ class fortran_intrinsic_obj(fortran_obj):
 
     def get_desc(self):
         if self.type == 2:
-            return 'SUBROUTINE'
+            return "SUBROUTINE"
         elif self.type == 14:
-            return 'KEYWORD'
+            return "KEYWORD"
         elif self.type == 15:
-            return 'STATEMENT'
+            return "STATEMENT"
         else:
-            return 'INTRINSIC'
+            return "INTRINSIC"
 
     def get_snippet(self, name_replace=None, drop_arg=-1):
         if self.args == "":
@@ -44,15 +54,7 @@ class fortran_intrinsic_obj(fortran_obj):
             arg_snip = None
         else:
             arg_list = self.args.split(",")
-            place_holders = []
-            for i, arg in enumerate(arg_list):
-                opt_split = arg.split("=")
-                if len(opt_split) > 1:
-                    place_holders.append("{1}=${{{0}:{2}}}".format(i+1, opt_split[0], opt_split[1]))
-                else:
-                    place_holders.append("${{{0}:{1}}}".format(i+1, arg))
-            arg_str = "({0})".format(", ".join(arg_list))
-            arg_snip = "({0})".format(", ".join(place_holders))
+            arg_str, arg_snip = self.get_placeholders(arg_list)
         name = self.name
         if name_replace is not None:
             name = name_replace
@@ -68,9 +70,6 @@ class fortran_intrinsic_obj(fortran_obj):
         call_sig, _ = self.get_snippet()
         return call_sig, self.doc_str, arg_sigs
 
-    def get_documentation(self):
-        return self.doc_str
-
     def get_hover(self, long=False):
         return self.doc_str, False
 
@@ -88,7 +87,7 @@ def load_intrinsics():
         if lowercase_intrinsics:
             name = name.lower()
             args = args.lower()
-        return fortran_intrinsic_obj(name, type, doc_str=doc_str, args=args)
+        return Intrinsic(name, type, doc_str=doc_str, args=args)
 
     def create_object(json_obj, enc_obj=None):
         if enc_obj is not None:
@@ -106,19 +105,26 @@ def load_intrinsics():
             name = name.lower()
             args = args.lower()
         if json_obj["type"] == 0:
-            mod_tmp = fortran_module(none_ast, 0, name)
+            mod_tmp = Module(none_ast, 0, name)
             if "use" in json_obj:
                 mod_tmp.add_use(json_obj["use"], 0)
             return mod_tmp
         elif json_obj["type"] == 1:
-            return fortran_subroutine(none_ast, 0, name, args=args)
+            return Subroutine(none_ast, 0, name, args=args)
         elif json_obj["type"] == 2:
-            return fortran_function(none_ast, 0, name,
-                                    args=args, return_type=[json_obj["return"], keywords, keyword_info])
+            return Function(
+                none_ast,
+                0,
+                name,
+                args=args,
+                result_type=json_obj["return"],
+                keywords=keywords,
+                # keyword_info=keyword_info,
+            )
         elif json_obj["type"] == 3:
-            return fortran_var(none_ast, 0, name, json_obj["desc"], keywords, keyword_info)
+            return Variable(none_ast, 0, name, json_obj["desc"], keywords, keyword_info)
         elif json_obj["type"] == 4:
-            return fortran_type(none_ast, 0, name, keywords)
+            return Type(none_ast, 0, name, keywords)
         else:
             raise ValueError
 
@@ -127,46 +133,47 @@ def load_intrinsics():
             child_obj = create_object(child, enc_obj=fort_obj)
             fort_obj.add_child(child_obj)
             add_children(child, child_obj)
+
     # Fortran statments taken from Intel Fortran documentation
-    # (https://software.intel.com/en-us/fortran-compiler-18.0-developer-guide)
-    json_file = os.path.join(os.path.dirname(os.path.abspath(__file__)), "statements.json")
-    statements = {
-        'var_def': [],
-        'int_stmnts': []
-    }
-    with open(json_file, 'r') as fid:
+    # (https://www.intel.com/content/www/us/en/develop/documentation/fortran-compiler-oneapi-dev-guide-and-reference/top/language-reference/a-to-z-reference)
+    json_file = os.path.join(
+        os.path.dirname(os.path.abspath(__file__)), "statements.json"
+    )
+    statements = {"var_def": [], "int_stmnts": []}
+    with open(json_file, "r", encoding="utf-8") as fid:
         intrin_file = json.load(fid)
         for key in statements:
             for name, json_obj in sorted(intrin_file[key].items()):
                 statements[key].append(create_int_object(name, json_obj, 15))
     # Fortran keywords taken from Intel Fortran documentation
-    # (https://software.intel.com/en-us/fortran-compiler-18.0-developer-guide)
-    json_file = os.path.join(os.path.dirname(os.path.abspath(__file__)), "keywords.json")
-    keywords = {
-        'var_def': [],
-        'arg': [],
-        'type_mem': [],
-        'vis': [],
-        'param': []
-    }
-    with open(json_file, 'r') as fid:
+    # (https://www.intel.com/content/www/us/en/develop/documentation/fortran-compiler-oneapi-dev-guide-and-reference/top/language-reference/a-to-z-reference)
+    json_file = os.path.join(
+        os.path.dirname(os.path.abspath(__file__)), "keywords.json"
+    )
+    keywords = {"var_def": [], "arg": [], "type_mem": [], "vis": [], "param": []}
+    with open(json_file, "r", encoding="utf-8") as fid:
         intrin_file = json.load(fid)
         for key in keywords:
             for name, json_obj in sorted(intrin_file[key].items()):
                 keywords[key].append(create_int_object(name, json_obj, 14))
     # Definitions taken from gfortran documentation
     # (https://gcc.gnu.org/onlinedocs/gfortran/Intrinsic-Procedures.html#Intrinsic-Procedures)
-    json_file = os.path.join(os.path.dirname(os.path.abspath(__file__)), "intrinsic_funs.json")
+    json_file = os.path.join(
+        os.path.dirname(os.path.abspath(__file__)), "intrinsic_funs.json"
+    )
     int_funs = []
-    with open(json_file, 'r') as fid:
+    with open(json_file, "r", encoding="utf-8") as fid:
         intrin_file = json.load(fid)
         for name, json_obj in sorted(intrin_file.items()):
-            int_funs.append(create_int_object(name, json_obj, json_obj['type']))
+            int_funs.append(create_int_object(name, json_obj, json_obj["type"]))
     # Definitions taken from gfortran documentation
     # (https://gcc.gnu.org/onlinedocs/gfortran/Intrinsic-Modules.html#Intrinsic-Modules)
-    json_file = os.path.join(os.path.dirname(os.path.abspath(__file__)), "intrinsic_mods.json")
+    # Update OpenACC from here https://www.openacc.org/specification
+    json_file = os.path.join(
+        os.path.dirname(os.path.abspath(__file__)), "intrinsic_mods.json"
+    )
     int_mods = []
-    with open(json_file, 'r') as fid:
+    with open(json_file, "r", encoding="utf-8") as fid:
         intrin_file = json.load(fid)
         for key, json_obj in intrin_file.items():
             fort_obj = create_object(json_obj)
@@ -177,11 +184,11 @@ def load_intrinsics():
 
 def get_intrinsic_keywords(statements, keywords, context=-1):
     if context == 0:
-        return statements['int_stmnts'] + statements['var_def'] + keywords['vis']
+        return statements["int_stmnts"] + statements["var_def"] + keywords["vis"]
     elif context == 1:
-        return keywords['var_def'] + keywords['vis'] + keywords['param']
+        return keywords["var_def"] + keywords["vis"] + keywords["param"]
     elif context == 2:
-        return keywords['var_def'] + keywords['arg'] + keywords['param']
+        return keywords["var_def"] + keywords["arg"] + keywords["param"]
     elif context == 3:
-        return keywords['var_def'] + keywords['type_mem'] + keywords['vis']
-    return keywords['var_def'] + keywords['param']
+        return keywords["var_def"] + keywords["type_mem"] + keywords["vis"]
+    return keywords["var_def"] + keywords["param"]
